# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['teleinforeader',
 'teleinforeader.database',
 'teleinforeader.io',
 'teleinforeader.model',
 'teleinforeader.util']

package_data = \
{'': ['*']}

install_requires = \
['mariadb==1.0.7', 'observable>=1.0.3,<2.0.0', 'pyserial>=3.5,<4.0']

entry_points = \
{'console_scripts': ['teleinforeader = teleinforeader.__main__:main']}

setup_kwargs = {
    'name': 'teleinforeader',
    'version': '0.1.0',
    'description': 'Application used to read the user data transmitted by Linky meter system (TeleInfo) from Enedis.',
    'long_description': '# TeleInfo Reader\n\nApplication used to read the user data transmitted by Linky meter system (TeleInfo) from Enedis. The application then\nprovides the data using a socket server to connected clients and also publish it to a local database. The application\nruns on a Raspberry Pi Zero and is used only to collect the user data from the meter equipment, the monitoring is done\nwith another application which connects to the Raspberry Pi and plot all the gathered information (see TeleInfoMonitor\napplication).\n\n## Hardware Description\n\n### Material\n\n* Raspberry Pi Zero\n  W ([Buy one](https://www.kubii.fr/les-cartes-raspberry-pi/1851-raspberry-pi-zero-w-kubii-3272496006997.html))\n* PiTInfo shield for Linky user\n  interface ([Buy one](https://www.tindie.com/products/Hallard/pitinfo/))\n\n### Raspberry Pi Zero\n\n#### Board version:\n\n```shell\n$ cat /proc/device-tree/model\nRaspberry Pi Zero W Rev 1.1\n```\n\n#### OS version:\n\n```shell\n$ cat /etc/os-release\nPRETTY_NAME="Raspbian GNU/Linux 10 (buster)"\n```\n\n## Serial Link Configuration\n\nThe acquisition of the user data frame (TeleInfo) is done using the serial link provided by Enedis meter equipment. On\nthe Raspberry Pi, the reception of the serial data is done using a hardware module called PiTInfo, which is connected on\nthe GPIOs of the Raspberry Pi board.\n\n### How to plug the PiTInfo module\n\nThe PiTInfo module should be plugged on GPIO pins {1..10} like in below picture:\n\n<img src="https://www.jonathandupre.fr/images/articles/2018/208/08.jpg" alt="image_pitinfo_plugged" style="width:400px;"/>\n\nFor information, the full mapping of GPIOs can be found here:\n[model-zerow-rev1](https://pi4j.com/1.2/pins/model-zerow-rev1.html)\n\nPin 8 and 10, respectively GPIO 15 and 16 corresponds to the UART, which means where we get the serial data from Linky\nmeter.\n\n### Serial Link Configuration\n\n* Disable serial console (in order to use the serial link device for Linky reception):\n    - Edit /boot/cmdline.txt\n    - Remove line: `console=serial0,(...)`\n* Disable bluetooth module(so it doesn\'t use UART):\n    - Edit /boot/config.txt\n    - Add line: `dtoverlay=pi3-miniuart-bt`\n* Reboot raspberry\n\n### How to test serial link\n\n```shell\n$ sudo stty -F /dev/ttyAMA0 1200 sane evenp parenb cs7 -crtscts\n$ sudo chmod 666 /dev/ttyAMA0\n$ sudo cat /dev/ttyAMA0\n```\n\nThis should output the data collected from Linky. If nothing returned, then serial link is probably not well configured.\n\n## How to create database on Raspberry\n\nThe application collects the TeleInfo frames and stores them into a local database called `teleinfodb`. This database\nshould be created and configured manually on the Raspberry Pi.\n\n### Install MariaDB server locally\n\n```shell\n$ sudo apt update\n$ sudo apt upgrade\n$ sudo apt install mariadb-server\n$ sudo apt install libmariadbclient-dev # For python bindings\n$ sudo apt install libmariadb3 libmariadb-dev # For python3 mariadb connectors\n```\n\n### Create and configure database\n\n#### Creation\n\nSecure and configure mysql:\n\n```shell\n$ sudo mysql_secure_installation # Say yes to all\n$ sudo mysql -u root -p\n```\n\nCreate database (replace `<user>`, `<password>` and `<remote-ip-address>` by your own):\n\n<i>NB: <remote-ip-address> is the IP address of the remote client from where it\'ll try to connect to the Raspberry\nPi.</i>\n\n```mariadb\nCREATE DATABASE teleinfodb;\nCREATE USER \'<user>\'@\'<remote-ip-address>\' IDENTIFIED BY \'<password>\';\nGRANT ALL PRIVILEGES ON teleinfodb.* TO \'<user>\'@\'<remote-ip-address>\';\nFLUSH PRIVILEGES;\n```\n\n#### Configuration\n\nEnable remote access to database (in order for a third party tool to connect to the database on the Raspberry Pi):\n\nIn file `/etc/mysql/mariadb.conf.d/50-server.cnf`, change line `bind-address = 127.0.0.1` to `bind-address = *`\n\nRestart mysql:\n\n```shell\n$ sudo service mysql restart\n```\n\nTo connect remotely:\n\n```shell\n$ mysql -u <user> -p -h <raspberry-ip-address>\n```\n\n#### Creating the DB schema and table\n\n```mariadb\ncreate schema teleinfodb;\nuse teleinfodb;\ncreate table teleinfoframes\n(\n    timestamp                    timestamp default current_timestamp() not null on update current_timestamp() comment \'Frame timestamp in format YYYY-MM-DD HH:mm:ss.zzzzzz\'\n        primary key,\n    meter_identifier             char(12)                              not null comment \'Identifier of Enedis telemeter\',\n    subscription_type            char(4)                               not null comment \'Customer subscription mode\',\n    subscription_power_in_a      tinyint unsigned                      not null comment \'Power in amperes\',\n    total_base_index_in_wh       int unsigned                          not null comment \'Total base index in W.h\',\n    current_pricing_period       char(4)                               not null comment \'Current period if using peak/off-peak pricing\',\n    instantaneous_intensity_in_a smallint unsigned                     not null comment \'Current intensity in amperes\',\n    intensity_max_in_a           smallint unsigned                     not null comment \'Maximum intensity in amperes\',\n    power_consumption_in_va      mediumint unsigned                    not null comment \'Power consumption in V.A\',\n    peak_off_peak_schedule       char                                  not null comment \'Peak/Off-peak time schedule\',\n    meter_state_code             char(6)                               null comment \'Error code returned by telemeter\'\n)\n    comment \'Table containing the teleinfo frames\' collate = utf8mb4_general_ci;\n```\n\n#### Get size of database\n\n```mariadb\nSELECT table_schema                                            \'teleinfodb\',\n       SUM(data_length + index_length)                         \'Size in Bytes\',\n       ROUND(SUM(data_length + index_length) / 1024 / 1024, 2) \'Size in MiB\'\nFROM information_schema.tables\nWHERE table_schema = \'teleinfodb\'\nGROUP BY table_schema;\n```\n\n## Project Management\n\n### How to publish project on Pypi\n\n- See [packaging-projects](https://packaging.python.org/en/latest/tutorials/packaging-projects/)\n- See [publishing-to-testpypi](https://py-pkgs.org/03-how-to-package-a-python#publishing-to-testpypi)\n\n#### Test publish using TestPyPi (from host):\n\nPre-requisites:\n\n* Create an account on TestPypi\n* Create file ~/.pypirc containing the PyPi API token\n\n```shell\n$ python -m pip install --upgrade pip setuptools wheel build\n$ python -m pip install --upgrade twine\n```\n\nPublishing:\n\n```shell\n$ poetry version patch # To increment the version number\n$ poetry build  # To generate distribution packages for the package\n$ poetry config repositories.test-pypi https://test.pypi.org/legacy/\n$ poetry publish --build -r test-pypi --username __token__ --password <api-token>\n```\n',
    'author': 'Julien LE SAUCE',
    'author_email': 'julien.lesauce@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jlesauce/TeleInfoReader',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
